	program ECE_Coulomb_1

c	Coulomb potential with spin connection, 
c     1D stationary approximation
c	Inspection of space variable (x) dependeny
c
c	Provides a solution for differential equation
c	d2Phi/dx2 + omega*dPhi/dx + domega/dx*Phi = rho/epsilon0

c Declarations
	implicit none
	integer ndim
	parameter (ndim=5000)
	double precision Phi(0:ndim), rho(0:ndim), omega(0:ndim), omegaD(0:ndim),
     * f, fmix, h, Phi0, Phi0D, xmax, kappa
	integer n, iout, nmax, r_type, o_type
	character*80 text
	logical converged, convtest

c Main program
	call input (text, iout, nmax, r_type, o_type ,Phi0, Phi0D, xmax, kappa, f, fmix, h)
	call init  (rho, omega, omegaD, ndim, nmax, r_type, o_type, kappa, xmax, h)

	if (o_type.eq.5) then
		! iterate to convergence
		do n=1,200
			call init5    (ndim, nmax, Phi, omega, omegaD, f, fmix, h)
			call iteration(ndim, nmax, Phi, rho, omega, omegaD, Phi0, Phi0D, f, h)
			call output5  (text, iout, ndim, nmax, Phi, xmax, n, 20)
			converged   = convtest(ndim, nmax, Phi, omega, f)
			if (converged) then
				print *, 'convergence after ', n, ' iterations.'
				goto 10
			endif
		enddo
		! no convergence achieved
		print *, 'WARNING: no convergence after ', n, ' iterations.'
	else
		! non-iterative solution
		call iteration(ndim, nmax, Phi, rho, omega, omegaD, Phi0, Phi0D, f, h)
	endif

10	call output (text, iout, ndim, nmax, Phi, rho, omega, omegaD, xmax)

	end

c-------------------------------------------------------------------------------	
	subroutine init(rho, omega, omegaD, ndim, nmax, r_type, o_type, kappa, xmax, h)
	implicit none
	integer nmax, ndim, r_type, o_type
	double precision rho(0:ndim), omega(0:ndim), omegaD(0:ndim), kappa, xmax, h

c	local variables
	integer n
	double precision x

c  Init current density rho
c  ========================
	if (r_type.eq.1) then
		! harmonic charge density
		do n=0,nmax
			x = real(n)/real(nmax) * xmax
			rho(n)=cos(kappa*x)
		enddo
	else if (r_type.eq.2) then
		! exponential oscillating charge density
		do n=0,nmax
			x = real(n)/real(nmax) * xmax
			rho(n)=exp(-(.1*kappa*x)**2)*cos(kappa*x)
		enddo
	else
		stop 'ERROR: r_type has wrong value.'
	endif

c  Init spin connection omega
c  ==========================
	if (o_type.eq.1) then
		! constant spin connection
		do n=0,nmax
			omega(n)=0.1
			!  Init spin connection derivative omega'
			omegaD(n)=2.
		enddo
	else if (o_type.eq.2) then
		! linear spin connection
		do n=0,nmax
			x = real(n)/real(nmax) * xmax
			omega(n)=0.1*x
		enddo	
		!  Init spin connection derivative omegaD
		do n=1,nmax-1
			omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
		enddo
		!  use asymmetric formulae for boundary points
		omegaD(0)=(omega(1)-omega(0))/h
		omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h
	else if (o_type.eq.3) then
		! harmonic spin connection
		do n=0,nmax
			x = real(n)/real(nmax) * xmax
			omega(n)=sin(kappa*x)
		enddo	
		!  Init spin connection derivative omegaD
		do n=1,nmax-1
			omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
		enddo
		!  use asymmetric formulae for boundary points
		omegaD(0)=(omega(1)-omega(0))/h
		omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h
	else if (o_type.eq.4) then
		! spin connection identical to rho
		do n=0,nmax
			omega(n)=rho(n)
		enddo	
		!  Init spin connection derivative omegaD
		do n=1,nmax-1
			omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
		enddo
		!  use asymmetric formulae for boundary points
		omegaD(0)=(omega(1)-omega(0))/h
		omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h
	else if (o_type.eq.5) then
		!  o_type.eq.5: solve nonlinear equation
		do n=0,nmax
			omega(n) =0.
			omegaD(n)=0.
		enddo	
	else
		stop 'ERROR: o_type has wrong value.'
	endif

	return
	end

c-------------------------------------------------------------------------------	
	subroutine init5(ndim, nmax, Phi, omega, omegaD, f, fmix, h)
	implicit none
	integer nmax, ndim
	double precision Phi(0:ndim), omega(0:ndim), omegaD(0:ndim), f, fmix, h

c	local variables
	integer n

c  Init spin connection omega
c  ==========================
	! spin connection identical to Phi, using mixing factor
	do n=0,nmax
		omega(n)=(1.d0-fmix)*omega(n)+fmix*f*Phi(n)
	enddo	
	!  Init spin connection derivative omegaD
	do n=1,nmax-1
		omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
	enddo
	!  use asymmetric formulae for boundary points
	omegaD(0)=(omega(1)-omega(0))/h
	omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h

	return
	end

c-------------------------------------------------------------------------------	
	logical function convtest(ndim, nmax, Phi, omega, f)
	implicit none
	integer ndim, nmax
	double precision Phi(0:ndim), omega(0:ndim), f
	
	integer n
	double precision delta
	delta = 1.d-4

	convtest = .true.
	do n=1,nmax-1
		if (dabs(f*Phi(n)-omega(n)).gt.delta) convtest=.false.
	enddo	
	return
	end

c-------------------------------------------------------------------------------	
	subroutine iteration (ndim, nmax, Phi, rho, omega, omegaD, Phi0, Phi0D, f, h)
	implicit none
	integer ndim, nmax, o_type
	double precision Phi(0:ndim), rho(0:ndim), omega(0:ndim), omegaD(0:ndim),
     * Phi0, Phi0D, f, h

c	local variables
	integer n
	double precision R

c  Initial values
	Phi(0) = Phi0
	!  second index of Phi is computed from derivative at Phi(0)
	Phi(1) = Phi(0) + h*Phi0D

c  Iteration
	!  solve equation with Phi, omega
	do n=1,nmax-1
		Phi(n+1) = 1.d0/(1.d0 + 0.5d0*h*omega(n)) * 
     *               (-Phi(n)*(-2.d0 + h*h*omegaD(n)) 
     *                - Phi(n-1)*(1.d0 - 0.5d0*h*omega(n))
     *                + h*h*rho(n))
	enddo
		
	end

c-------------------------------------------------------------------------------	
	subroutine input (text, iout, nmax, r_type, o_type ,Phi0, Phi0D, xmax, kappa, f, fmix, h)
	implicit none
	character*80 text
	integer iout, r_type, o_type, nmax
	double precision Phi0, Phi0D, xmax, kappa, f, fmix, h
	
	open (10, name='input1.txt', status='old')
	read (10,'(a)') text
	read (10,*) iout
	read (10,*) nmax, xmax
	read (10,*) r_type, o_type
	read (10,*) Phi0, Phi0D
	read (10,*) kappa
	read (10,*) f, fmix
	close (10)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	! dynamic xmax setting
	! restricting to 3 full oscillations
	if (xmax.lt.0.) xmax = 3*2*3.141/kappa
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	print *, 'Input Data:'
	print *, text
	print *, 'iout           = ', iout
	print *, 'nmax, xmax     = ', nmax, xmax
	print *, 'r_type, o_type = ', r_type, o_type
	print *, 'Phi0, Phi0D, f = ', Phi0, Phi0D, f
	print *, 'kappa          = ', kappa
	print *, 'f, fmix        = ', f, fmix

	if (iout.ge.2) then
		print *, '---------------------------------------------------------------------------'
		print *, '   n      x             Phi           rho           omega          omegaD         Q'
		print *, '---------------------------------------------------------------------------'
	end if

	h = 1.d0/nmax*xmax

	! truncate file output1a.txt
	! this is required to avoid false graph output in case o_type < 5	
	open (20, name='output1a.txt', status='replace')
	close(20)
	end

c-------------------------------------------------------------------------------	
	subroutine output (text, iout, ndim, nmax, Phi, rho, omega, omegaD, xmax)
	implicit none
	character*80 text
	integer iout, ndim, nmax
	double precision Phi(0:ndim), rho(0:ndim), omega(0:ndim), omegaD(0:ndim), xmax

c	local variables
	integer n
	double precision x, Q

	open (20, name='output1.txt', status='replace')
	write (20,'(2a)') "#", text
	write (20,'(a)')  '#  n      x             Phi           rho           omega          omegaD         Q'

c write Phi etc.
	do n=0,nmax
		x = real(n)/real(nmax) * xmax
		! compute Q factor (special definition)
		if (dabs(omega(n)).gt.1000.) then
			Q = sign(1., omega(n)) * 1000.
		else if (dabs(omega(n)).lt.1.d-8) then
			Q = 0.d0
		else
			Q = omegaD(n)/omega(n)
		endif
		if (iout.ge.2) write (*,100) n, x, Phi(n), rho(n), omega(n), omegaD(n), Q
		write (20,100) n, x, Phi(n), rho(n), omega(n), omegaD(n), Q
	end do
	close (20)

100	format(i5,f14.6,e14.6,9f14.6)
	end

c-------------------------------------------------------------------------------	
	subroutine output5 (text, iout, ndim, nmax, Phi, xmax, it, itmax)
	implicit none
	character*80 text
	integer iout, ndim, nmax, it, itmax
	double precision Phi(0:ndim), xmax

c	local variables
	integer n
	double precision x, Q

	if (it.eq.1) then
		open (20, name='output1a.txt', status='replace')
		write (20,'(2a)') "#", text
		write (20,'(a)')  '#  n      x             Phi'
	endif

c write Phi etc.
	do n=0,nmax
		x = real(n)/real(nmax) * xmax
		if (iout.ge.2) write (*,100) n, x, Phi(n)
		write (20,100) n, x, Phi(n)
	end do
	! write empty line to mark block border
	write (20,*)
	if (it.eq.itmax) close (20)

100	format(i5,f14.6,e14.6,9f14.6)
	end
