	program ECE_Coulomb_2

c	Coulomb potential with spin connection, 
c     1D stationary approximation
c	Inspection of kappa dependency
c
c	Provides a solution for differential equation
c	d2Phi/dx2 + omega*dPhi/dx + domega/dx*Phi = rho/epsilon0

c Declarations
	implicit none
	integer ndim
	parameter (ndim=5000)
	double precision Phi(0:ndim), rho(0:ndim), omega(0:ndim), omegaD(0:ndim),
     * deltaPhi(0:ndim), h, Phi0, Phi0D, xmax, xmax_in, kappa, kappamin,
     * kappamax
	integer n, nk, iout, nmax, nkmax, r_type, o_type
	character*80 text

c Main program
	call input (text, iout, nmax, r_type, o_type ,Phi0, Phi0D, xmax_in, h,
     * nkmax, kappamin, kappamax)
	do nk=0,nkmax
		kappa = real(nk)/real(nkmax) * (kappamax-kappamin) + kappamin
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		! dynamic xmax setting
		! restricting to 3 full oscillations
		if (xmax_in.lt.0.) then
			xmax = 3*2*3.141/kappa
		else
			xmax = xmax_in
		end if
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		h = 1.d0/nmax*xmax
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		call init     (rho, omega, omegaD, ndim, nmax, r_type, o_type, kappa, xmax, h)
		call iteration(ndim, nmax, o_type, Phi, rho, omega, omegaD, Phi0, Phi0D, h)
		call findmax  (ndim, nmax, Phi, deltaPhi(nk))
		if (deltaPhi(nk).eq.0.) print *,'Warning: deltaPhi=0 for nk, kappa =', nk, kappa

	enddo
	call output (text, iout, ndim, nkmax, deltaPhi, omega, omegaD, kappamin, kappamax)
	end

c-------------------------------------------------------------------------------	
	subroutine init(rho, omega, omegaD, ndim, nmax, r_type, o_type, kappa, xmax, h)
	implicit none
	integer nmax, ndim, r_type, o_type
	double precision rho(0:ndim), omega(0:ndim), omegaD(0:ndim), kappa, xmax, h

c	local variables
	integer n
	double precision x

c  Init current density rho
c  ========================
	if (r_type.eq.1) then
		! harmonic charge density
		do n=0,nmax
			x = real(n)/real(nmax) * xmax
			rho(n)=cos(kappa*x)
		enddo
	else if (r_type.eq.2) then
		! exponential oscillating charge density
		do n=0,nmax
			x = real(n)/real(nmax) * xmax
			rho(n)=exp(-(.1*kappa*x)**2)*cos(kappa*x)
		enddo
	else
		stop 'ERROR: r_type has wrong value.'
	endif

c  Init spin connection omega
c  ==========================
	if (o_type.eq.1) then
		! constant spin connection
		do n=0,nmax
			omega(n)=0.1
		enddo
		!  Init spin connection derivative omega'
		do n=0,nmax
			omegaD(n)=2.
		enddo
	else if (o_type.eq.2) then
		! linear spin connection
		do n=0,nmax
			x = real(n)/real(nmax) * xmax
			omega(n)=0.1*x
		enddo	
		!  Init spin connection derivative omegaD
		do n=1,nmax-1
			omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
		enddo
		!  use asymmetric formulae for boundary points
		omegaD(0)=(omega(1)-omega(0))/h
		omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h
	else if (o_type.eq.3) then
		! harmonic spin connection
		do n=0,nmax
			x = real(n)/real(nmax) * xmax
			omega(n)=sin(kappa*x)
		enddo	
		!  Init spin connection derivative omegaD
		do n=1,nmax-1
			omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
		enddo
		!  use asymmetric formulae for boundary points
		omegaD(0)=(omega(1)-omega(0))/h
		omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h
	else if (o_type.eq.4) then
		! spin connection identical to rho
		do n=0,nmax
			omega(n)=rho(n)
		enddo	
		!  Init spin connection derivative omegaD
		do n=1,nmax-1
			omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
		enddo
		!  use asymmetric formulae for boundary points
		omegaD(0)=(omega(1)-omega(0))/h
		omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h
	else if (o_type.eq.5) then
		!  o_type.eq.5: solve nonlinear equation
		do n=0,nmax
			omega(n) =0.
			omegaD(n)=0.
		enddo	
	else
		stop 'ERROR: o_type has wrong value.'
	endif

	return
	end

c-------------------------------------------------------------------------------	
	subroutine iteration (ndim, nmax, o_type, Phi, rho, omega, omegaD, Phi0, Phi0D, h)
	implicit none
	integer ndim, nmax, o_type
	double precision Phi(0:ndim), rho(0:ndim), omega(0:ndim), omegaD(0:ndim),
     * Phi0, Phi0D, h

c	local variables
	integer n
	double precision f

	f = .1d0

c  Initial values
	Phi(0) = Phi0
	!  second index of Phi is computed from derivative at Phi(0)
	Phi(1) = Phi(0) + h*Phi0D

	if (o_type.ne.5) then
		!  solve equation with Phi, omega
		do n=1,nmax-1
			Phi(n+1) = 1.d0/(1.d0 + 0.5d0*h*omega(n)) * 
     *               (-Phi(n)*(-2.d0 + h*h*omegaD(n)) 
     *                - Phi(n-1)*(1.d0 - 0.5d0*h*omega(n))
     *                + h*h*rho(n))
		enddo
	else
		!  o_type.eq.5: solve nonlinear equation
		do n=1,nmax-1
			Phi(n+1) = 1.d0/(1.d0 + f*h*Phi(n)) * 
     *               (Phi(n)*(2.d0 + f*h*Phi(n-1)) 
     *                - Phi(n-1)
     *                + h*h*rho(n))
		enddo
	endif
	end

c-------------------------------------------------------------------------------	
	subroutine findmax (ndim, nmax, Phi, deltaPhi)
	implicit none
	integer ndim, nmax
	double precision Phi(0:ndim), deltaPhi

c	local variables
	integer n
	double precision phimin, phimax
	logical initialized

	phimin= 1.d23
	phimax=-1.d23
	initialized=.false.
	deltaPhi = 0.
c  Find maximum difference between all neighbouring max/min pairs of Phi
	do n=nmax/3,nmax-1
		if (phi(n).gt.phi(n-1).and.phi(n).gt.phi(n+1)) then
			! this is a maximum
			phimax=phi(n)
c	if (abs(kappa-8.004).le.0.1) print *,'max:',phimax
			if (initialized) deltaPhi = max(deltaPhi, phimax-phimin)
			initialized=.true.
		else if (phi(n).lt.phi(n-1).and.phi(n).lt.phi(n+1)) then
			! this is a minimum
			phimin=phi(n)
c	if (abs(kappa-8.004).le.0.1) print *,'min:',phimin
			if (initialized) deltaPhi = max(deltaPhi, phimax-phimin)
			initialized=.true.
		endif
	enddo
	
c	if (abs(kappa-8.004).le.0.1) print *,kappa,' ####### ',deltaPhi
c	if (deltaPhi.eq.0.) print *,'######################'

	return
	end

c-------------------------------------------------------------------------------	
	subroutine input (text, iout, nmax, r_type, o_type ,Phi0, Phi0D, xmax, h,
     * nkmax, kappamin, kappamax)
	implicit none
	character*80 text
	integer iout, r_type, o_type, nmax, nkmax
	double precision Phi0, Phi0D, xmax, kappamin, kappamax, h
	
	open (10, name='input2.txt', status='old')
	read (10,'(a)') text
	read (10,*) iout
	read (10,*) nmax, xmax
	read (10,*) r_type, o_type
	read (10,*) Phi0, Phi0D
	read (10,*) nkmax
	read (10,*) kappamin, kappamax
	close (10)

	print *, 'Input Data:'
	print *, text
	print *, 'iout          = ', iout
	print *, 'nmax, xmax    = ', nmax, xmax
	print *, 'r_type, o_type= ', r_type, o_type
	print *, 'Phi0, Phi0D   = ', Phi0, Phi0D
	print *, 'nkmax         = ', nkmax
	print *, 'kappamin,max  = ', kappamin, kappamax

	if (iout.ge.2) then
		print *, '---------------------------------------------------------------------------'
		print *, '   nk     kappa         deltaPhi'
		print *, '---------------------------------------------------------------------------'
	end if

	end

c-------------------------------------------------------------------------------	
	subroutine output (text, iout, ndim, nkmax, deltaPhi, omega_k, omegaD_k, kappamin, kappamax)
	implicit none
	character*80 text
	integer iout, ndim, nkmax
	double precision deltaPhi(0:ndim), omega_k(0:ndim), omegaD_k(0:ndim), kappamin, kappamax

c	local variables
	integer nk
	double precision kappa, kappa_res, deltaphi_max

	deltaphi_max = 0.d0

	open (20, name='output2.txt', status='replace')
	write (20,'(2a)') "#", text
	write (20,'(a)')  '#  nk     kappa         deltaPhi'

c write deltaPhi etc.
	do nk=0,nkmax
		kappa = real(nk)/real(nkmax) * (kappamax-kappamin) + kappamin
		if (deltaPhi(nk).gt.deltaphi_max) then
			deltaphi_max = deltaPhi(nk)
			kappa_res   = kappa
		endif
		if (iout.ge.2) write (*,100) nk, kappa, deltaPhi(nk)
		write (20,100) nk, kappa, deltaPhi(nk)
	end do
	close (20)

	write (*,*)
	write (*,*) 'Resonance at kappa  = ', kappa_res
	write (*,*) 'Resonance amplitude = ', deltaphi_max

100	format(i5,f14.6,e14.6,9f14.6)
	end
