	program ECE_Coulomb_1

c	Coulomb potential with spin connection, 
c     1D stationary approximation
c	Inspection of space variable (x) dependeny
c
c	Provides a solution for differential equation
c	d2Phi/dx2 + omega*dPhi/dx + domega/dx*Phi = rho/epsilon0

c Declarations
	implicit none
	integer ndim
	parameter (ndim=5000)
	double precision Phi(0:ndim), rho(0:ndim), omega(0:ndim), omegaD(0:ndim),
     * f, fmix, h, Phi0, Phi0D, xmax, kappa
	integer n, iout, nmax, r_type, o_type
	character*80 text

c Main program
	call input    (text, iout, nmax, r_type, o_type ,Phi0, Phi0D, xmax, kappa, f, fmix, h)
	call init     (rho, omega, omegaD, ndim, nmax, r_type, o_type, kappa, xmax, h)
	do n=1,10
		call init5    (Phi, omega, omegaD, f, ndim, nmax, h)
		call iteration(ndim, nmax, Phi, rho, omega, omegaD, Phi0, Phi0D, f, h)
		call output5  (text, iout, ndim, nmax, Phi, rho, omega, omegaD, xmax, (n+1)/2)
	enddo
	end

c-------------------------------------------------------------------------------	
	subroutine init(rho, omega, omegaD, ndim, nmax, r_type, o_type, kappa, xmax, h)
	implicit none
	integer nmax, ndim, r_type, o_type
	double precision rho(0:ndim), omega(0:ndim), omegaD(0:ndim), kappa, xmax, h

c	local variables
	integer n
	double precision x

c  Init current density rho
c  ========================
	if (r_type.eq.1) then
		! harmonic charge density
		do n=0,nmax
			x = dreal(n)/dreal(nmax) * xmax
			rho(n)=cos(kappa*x)
		enddo
	else if (r_type.eq.2) then
		! exponential oscillating charge density
		do n=0,nmax
			x = dreal(n)/dreal(nmax) * xmax
			rho(n)=exp(-(.1*kappa*x)**2)*cos(kappa*x)
		enddo
	endif

c  Init spin connection omega
c  ==========================
	if (o_type.eq.1) then
		! constant spin connection
		do n=0,nmax
			omega(n)=0.1
			!  Init spin connection derivative omega'
			omegaD(n)=2.
		enddo
	else if (o_type.eq.2) then
		! linear spin connection
		do n=0,nmax
			x = dreal(n)/dreal(nmax) * xmax
			omega(n)=0.1*x
		enddo	
		!  Init spin connection derivative omegaD
		do n=1,nmax-1
			omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
		enddo
		!  use asymmetric formulae for boundary points
		omegaD(0)=(omega(1)-omega(0))/h
		omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h
	else if (o_type.eq.3) then
		! harmonic spin connection
		do n=0,nmax
			x = dreal(n)/dreal(nmax) * xmax
			omega(n)=sin(kappa*x)
		enddo	
		!  Init spin connection derivative omegaD
		do n=1,nmax-1
			omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
		enddo
		!  use asymmetric formulae for boundary points
		omegaD(0)=(omega(1)-omega(0))/h
		omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h
	else if (o_type.eq.4.or.o_type.eq.5) then
		! spin connection identical to rho
		do n=0,nmax
			omega(n)=rho(n)
		enddo	
		!  Init spin connection derivative omegaD
		do n=1,nmax-1
			omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
		enddo
		!  use asymmetric formulae for boundary points
		omegaD(0)=(omega(1)-omega(0))/h
		omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h
	endif

	return
	end

c-------------------------------------------------------------------------------	
	subroutine init5(Phi, omega, omegaD, f, ndim, nmax, h)
	implicit none
	integer nmax, ndim
	double precision Phi(0:ndim), omega(0:ndim), omegaD(0:ndim), f, h

c	local variables
	integer n
	double precision fmix
	fmix = .3d0

c  Init spin connection omega
c  ==========================
	! spin connection identical to Phi, using mixing factor
	do n=0,nmax
		omega(n)=(1.d0-fmix)*omega(n)+fmix*f*Phi(n)
	enddo	
	!  Init spin connection derivative omegaD
	do n=1,nmax-1
		omegaD(n)=(omega(n+1)-omega(n-1))/(2.*h)
	enddo
	!  use asymmetric formulae for boundary points
	omegaD(0)=(omega(1)-omega(0))/h
	omegaD(nmax)=(omega(nmax)-omega(nmax-1))/h

	return
	end

c-------------------------------------------------------------------------------	
	subroutine iteration (ndim, nmax, Phi, rho, omega, omegaD, Phi0, Phi0D, f, h)
	implicit none
	integer ndim, nmax
	double precision Phi(0:ndim), rho(0:ndim), omega(0:ndim), omegaD(0:ndim),
     * Phi0, Phi0D, f, h

c	local variables
	integer n
	double precision R

c  Initial values
	Phi(0) = Phi0
	!  second index of Phi is computed from derivative at Phi(0)
	Phi(1) = Phi(0) + h*Phi0D

c  Iteration
	!  solve equation with Phi, omega
	do n=1,nmax-1
		Phi(n+1) = 1.d0/(1.d0 + 0.5d0*h*omega(n)) * 
     *               (-Phi(n)*(-2.d0 + h*h*omegaD(n)) 
     *                - Phi(n-1)*(1.d0 - 0.5d0*h*omega(n))
     *                + h*h*rho(n))
	enddo
		
	end

c-------------------------------------------------------------------------------	
	subroutine input (text, iout, nmax, r_type, o_type ,Phi0, Phi0D, xmax, kappa, f, fmix, h)
	implicit none
	character*80 text
	integer iout, r_type, o_type, nmax
	double precision Phi0, Phi0D, xmax, kappa, f, fmix, h
	
	open (10, name='input1.txt', status='old')
	read (10,'(a)') text
	read (10,*) iout
	read (10,*) nmax, xmax
	read (10,*) r_type, o_type
	read (10,*) Phi0, Phi0D
	read (10,*) kappa
	read (10,*) f, fmix
	close (10)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	! dynamic xmax setting
	! restricting to 3 full oscillations
	if (xmax.lt.0.) xmax = 3*2*3.141/kappa
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	print *, 'Input Data:'
	print *, text
	print *, 'iout           = ', iout
	print *, 'nmax, xmax     = ', nmax, xmax
	print *, 'r_type, o_type = ', r_type, o_type
	print *, 'Phi0, Phi0D, f = ', Phi0, Phi0D, f
	print *, 'kappa          = ', kappa
	print *, 'f, fmix        = ', kappa

	if (iout.ge.2) then
		print *, '---------------------------------------------------------------------------'
		print *, '   n      x             Phi           rho           omega          omegaD         Q'
		print *, '---------------------------------------------------------------------------'
	end if

	h = 1.d0/nmax*xmax
	end

c-------------------------------------------------------------------------------	
	subroutine output5 (text, iout, ndim, nmax, Phi, rho, omega, omegaD, xmax, it)
	implicit none
	character*80 text
	integer iout, ndim, nmax, it
	double precision Phi(0:ndim), rho(0:ndim), omega(0:ndim), omegaD(0:ndim), xmax

c	local variables
	integer n
	double precision x, Q
	character*20 outfile(5)
	data outfile /'output1.txt','output2.txt','output3.txt','output4.txt','output5.txt'/

	open (20, name=outfile(it), status='replace')
	write (20,'(2a)') "#", text
	write (20,'(a)')  '#  n      x             Phi'

c write Phi etc.
	do n=0,nmax
		x = dreal(n)/dreal(nmax) * xmax
		! compute Q factor (special definition)
		if (dabs(omega(n)).gt.1000.) then
			Q = sign(1., omega(n)) * 1000.
		else if (dabs(omega(n)).lt.1.d-8) then
			Q = 0.d0
		else
			Q = omegaD(n)/omega(n)
		endif
		if (iout.ge.2) write (*,100) n, x, Phi(n)
		write (20,100) n, x, Phi(n)
	end do
	close (20)

100	format(i5,f14.6,e14.6,9f14.6)
	end